@@ -32,7 +32,7 @@
 #include <libedata-book/e-data-book-view.h>
 #include "e-book-backend-file.h"
 
-#define d(x)
+#define d(x) x
 
 #define CHANGES_DB_SUFFIX ".changes.db"
 
@@ -157,6 +157,7 @@ do_create(EBookBackendFile  *bf,
 static EBookBackendSyncStatus
 e_book_backend_file_create_contact (EBookBackendSync *backend,
 				    EDataBook *book,
+				    guint32 opid,
 				    const char *vcard,
 				    EContact **contact)
 {
@@ -176,7 +177,8 @@ e_book_backend_file_create_contact (EBookBackendSync *backend,
 
 static EBookBackendSyncStatus
 e_book_backend_file_remove_contacts (EBookBackendSync *backend,
-				     EDataBook    *book,
+				     EDataBook *book,
+				     guint32 opid,
 				     GList *id_list,
 				     GList **ids)
 {
@@ -222,7 +224,8 @@ e_book_backend_file_remove_contacts (EBookBackendSync *backend,
 
 static EBookBackendSyncStatus
 e_book_backend_file_modify_contact (EBookBackendSync *backend,
-				    EDataBook    *book,
+				    EDataBook *book,
+				    guint32 opid,
 				    const char *vcard,
 				    EContact **contact)
 {
@@ -276,7 +279,8 @@ e_book_backend_file_modify_contact (EBookBackendSync *backend,
 
 static EBookBackendSyncStatus
 e_book_backend_file_get_contact (EBookBackendSync *backend,
-				 EDataBook    *book,
+				 EDataBook *book,
+				 guint32 opid,
 				 const char *id,
 				 char **vcard)
 {
@@ -304,7 +308,8 @@ e_book_backend_file_get_contact (EBookBackendSync *backend,
 
 static EBookBackendSyncStatus
 e_book_backend_file_get_contact_list (EBookBackendSync *backend,
-				      EDataBook    *book,
+				      EDataBook *book,
+				      guint32 opid,
 				      const char *query,
 				      GList **contacts)
 {
@@ -364,7 +369,10 @@ e_book_backend_file_get_contact_list (EBookBackendSync *backend,
 }
 
 typedef struct {
+	EBookBackendFile *bf;
 	GMutex *mutex;
+	GCond *cond;
+	GThread *thread;
 	gboolean stopped;
 } FileBackendSearchClosure;
 
@@ -373,15 +381,19 @@ closure_destroy (FileBackendSearchClosure *closure)
 {
 	d(printf ("destroying search closure\n"));
 	g_mutex_free (closure->mutex);
+	g_cond_free (closure->cond);
 	g_free (closure);
 }
 
 static FileBackendSearchClosure*
-init_closure (EDataBookView *book_view)
+init_closure (EDataBookView *book_view, EBookBackendFile *bf)
 {
 	FileBackendSearchClosure *closure = g_new (FileBackendSearchClosure, 1);
 
-	closure->mutex = g_mutex_new();
+	closure->bf = bf;
+	closure->mutex = g_mutex_new ();
+	closure->cond = g_cond_new ();
+	closure->thread = NULL;
 	closure->stopped = FALSE;
 
 	g_object_set_data_full (G_OBJECT (book_view), "EBookBackendFile.BookView::closure",
@@ -396,12 +408,12 @@ get_closure (EDataBookView *book_view)
 	return g_object_get_data (G_OBJECT (book_view), "EBookBackendFile.BookView::closure");
 }
 
-static void
-e_book_backend_file_start_book_view (EBookBackend  *backend,
-				     EDataBookView *book_view)
+static gpointer
+book_view_thread (gpointer data)
 {
-	FileBackendSearchClosure *closure;
-	EBookBackendFile *bf = E_BOOK_BACKEND_FILE (backend);
+	EDataBookView *book_view = data;
+	FileBackendSearchClosure *closure = get_closure (book_view);
+	EBookBackendFile *bf = closure->bf;
 	const char *query;
 	DB  *db;
 	DBT id_dbt, vcard_dbt;
@@ -410,30 +422,9 @@ e_book_backend_file_start_book_view (EBookBackend  *backend,
 
 	d(printf ("starting initial population of book view\n"));
 
-	g_mutex_lock (e_data_book_view_get_mutex (book_view));
-
-	closure = get_closure (book_view);
-	if (closure) {
-		/* the only way op can be set here is if we raced with
-		   stop_book_view and lost.  make sure by checking
-		   op->stopped. */
-		if (!closure->stopped) {
-			g_warning ("lost race with stop_book_view, but op->stopped != TRUE");
-		}
-		g_object_set_data (G_OBJECT (book_view), "EBookBackendFile.BookView::closure", NULL);
-		g_mutex_unlock (e_data_book_view_get_mutex (book_view));
-		return;
-	}
-	else {
-		closure = init_closure (book_view);
-		closure->stopped = FALSE;
-	}
-
 	/* ref the book view because it'll be removed and unrefed
 	   when/if it's stopped */
 	bonobo_object_ref (book_view);
-
-	g_mutex_unlock (e_data_book_view_get_mutex (book_view));
 	
 	db = bf->priv->file_db;
 	query = e_data_book_view_get_card_query (book_view);
@@ -443,6 +434,11 @@ e_book_backend_file_start_book_view (EBookBackend  *backend,
 	else
 		e_data_book_view_notify_status_message (book_view, _("Searching..."));
 
+	d(printf ("signalling parent thread\n"));
+	g_mutex_lock (closure->mutex);
+	g_cond_signal (closure->cond);
+	g_mutex_unlock (closure->mutex);
+
 	if (e_book_backend_summary_is_summary_query (bf->priv->summary, query)) {
 		/* do a summary query */
 		GPtrArray *ids = e_book_backend_summary_search (bf->priv->summary, e_data_book_view_get_card_query (book_view));
@@ -516,32 +512,45 @@ e_book_backend_file_start_book_view (EBookBackend  *backend,
 	/* unref the */
 	bonobo_object_unref (book_view);
 
-	d(printf ("finished initial population of book view\n"));
+	d(printf ("finished population of book view\n"));
+
+	return NULL;
+}
+
+static void
+e_book_backend_file_start_book_view (EBookBackend  *backend,
+				     EDataBookView *book_view)
+{
+	FileBackendSearchClosure *closure = init_closure (book_view, E_BOOK_BACKEND_FILE (backend));
+
+	g_mutex_lock (closure->mutex);
+
+	d(printf ("starting book view thread\n"));
+	closure->thread = g_thread_create (book_view_thread, book_view, TRUE, NULL);
+
+	g_cond_wait (closure->cond, closure->mutex);
+
+	/* at this point we know the book view thread is actually running */
+	g_mutex_unlock (closure->mutex);
+	d(printf ("returning from start_book_view\n"));
 }
 
 static void
 e_book_backend_file_stop_book_view (EBookBackend  *backend,
 				    EDataBookView *book_view)
 {
-	FileBackendSearchClosure *closure;
+	FileBackendSearchClosure *closure = get_closure (book_view);
+	gboolean need_join = FALSE;
 
-	g_mutex_lock (e_data_book_view_get_mutex (book_view));
+	d(printf ("stopping query\n"));
+	g_mutex_lock (closure->mutex);
+	if (!closure->stopped)
+		need_join = TRUE;
+	closure->stopped = TRUE;
+	g_mutex_unlock (closure->mutex);
 
-	closure = get_closure (book_view);
-
-	if (closure) {
-		d(printf ("stopping running query!\n"));
-		g_mutex_lock (closure->mutex);
-		closure->stopped = TRUE;
-		g_mutex_unlock (closure->mutex);
-	}
-	else {
-		d(printf ("either the query is already finished or hasn't started yet (we won the race)\n"));
-		closure = init_closure (book_view);
-		closure->stopped = TRUE;
-	}
-
-	g_mutex_unlock (e_data_book_view_get_mutex (book_view));
+	if (need_join)
+		g_thread_join (closure->thread);
 }
 
 typedef struct {
@@ -588,7 +597,8 @@ e_book_backend_file_changes_foreach_key (const char *key, gpointer user_data)
 
 static EBookBackendSyncStatus
 e_book_backend_file_get_changes (EBookBackendSync *backend,
-				 EDataBook    *book,
+				 EDataBook *book,
+				 guint32 opid,
 				 const char *change_id,
 				 GList **changes_out)
 {
@@ -729,7 +739,8 @@ e_book_backend_file_extract_path_from_uri (const char *uri)
 
 static EBookBackendSyncStatus
 e_book_backend_file_authenticate_user (EBookBackendSync *backend,
-				       EDataBook    *book,
+				       EDataBook *book,
+				       guint32 opid,
 				       const char *user,
 				       const char *passwd,
 				       const char *auth_method)
@@ -739,7 +750,8 @@ e_book_backend_file_authenticate_user (EBookBackendSync *backend,
 
 static EBookBackendSyncStatus
 e_book_backend_file_get_supported_fields (EBookBackendSync *backend,
-					  EDataBook    *book,
+					  EDataBook *book,
+					  guint32 opid,
 					  GList **fields_out)
 {
 	GList *fields = NULL;
@@ -880,7 +892,9 @@ e_book_backend_file_maybe_upgrade_db (EBookBackendFile *bf)
 	return ret_val;
 }
 
-#include <libedata-book/ximian-vcard.h>
+#ifdef CREATE_DEFAULT_VCARD
+# include <libedata-book/ximian-vcard.h>
+#endif
 
 static GNOME_Evolution_Addressbook_CallStatus
 e_book_backend_file_load_source (EBookBackend           *backend,
@@ -953,11 +967,13 @@ e_book_backend_file_load_source (EBookBackend           *backend,
 				g_warning ("db->open (... DB_CREATE ...) failed with %d", db_error);
 			}
 			else {
+#ifdef CREATE_DEFAULT_VCARD
 				EContact *contact;
 
 				contact = do_create(bf, XIMIAN_VCARD);
 				/* XXX check errors here */
 				g_object_unref (contact);
+#endif
 
 				writable = TRUE;
 			}
@@ -1024,7 +1040,8 @@ select_changes (const char *name)
 
 static EBookBackendSyncStatus
 e_book_backend_file_remove (EBookBackendSync *backend,
-			    EDataBook        *book)
+			    EDataBook        *book,
+			    guint32           opid)
 {
 	EBookBackendFile *bf = E_BOOK_BACKEND_FILE (backend);
 	GDir *dir;
@@ -1039,7 +1056,7 @@ e_book_backend_file_remove (EBookBackendSync *backend,
 	/* unref the summary before we remove the file so it's not written out again */
 	g_object_unref (bf->priv->summary);
 	bf->priv->summary = NULL;
-	if (-1 == unlink (bf->priv->filename))
+	if (-1 == unlink (bf->priv->summary_filename))
 		g_warning ("failed to remove summary file `%s`: %s", bf->priv->summary_filename, strerror (errno));
 
 	dir = g_dir_open (bf->priv->dirname, 0, NULL);
