@@ -20,6 +20,8 @@
 
 #include <libgnome/gnome-i18n.h>
 #include <libebook/e-contact.h>
+
+#include <libedataserver/e-util.h>
  
 #include <libedata-book/e-data-book.h>
 #include <libedata-book/e-data-book-view.h>
@@ -29,6 +31,8 @@
 #define PAS_ID_PREFIX "pas-id-"
 #define FILE_FLUSH_TIMEOUT 5000
 
+#define d(x)
+
 static EBookBackendSyncClass *e_book_backend_vcf_parent_class;
 typedef struct _EBookBackendVCFBookView EBookBackendVCFBookView;
 typedef struct _EBookBackendVCFSearchContext EBookBackendVCFSearchContext;
@@ -51,63 +55,6 @@ e_book_backend_vcf_create_unique_id ()
 	return g_strdup_printf (PAS_ID_PREFIX "%08lX%08X", time(NULL), c++);
 }
 
-typedef struct {
-	EBookBackendVCF *bvcf;
-	EDataBook *book;
-	EDataBookView *view;
-} VCFBackendSearchClosure;
-
-static void
-free_search_closure (VCFBackendSearchClosure *closure)
-{
-	g_free (closure);
-}
-
-static void
-foreach_search_compare (char *id, char *vcard_string, VCFBackendSearchClosure *closure)
-{
-	EContact *contact;
-
-	contact = e_contact_new_from_vcard (vcard_string);
-	e_data_book_view_notify_update (closure->view, contact);
-	g_object_unref (contact);
-}
-
-static gboolean
-e_book_backend_vcf_search_timeout (gpointer data)
-{
-	VCFBackendSearchClosure *closure = data;
-
-	g_hash_table_foreach (closure->bvcf->priv->contacts,
-			      (GHFunc)foreach_search_compare,
-			      closure);
-
-	e_data_book_view_notify_complete (closure->view, GNOME_Evolution_Addressbook_Success);
-
-	free_search_closure (closure);
-
-	return FALSE;
-}
-
-
-static void
-e_book_backend_vcf_search (EBookBackendVCF  	      *bvcf,
-			   EDataBookView           *book_view)
-{
-	const char *query = e_data_book_view_get_card_query (book_view);
-	VCFBackendSearchClosure *closure = g_new0 (VCFBackendSearchClosure, 1);
-
-	if ( ! strcmp (query, "(contains \"x-evolution-any-field\" \"\")"))
-		e_data_book_view_notify_status_message (book_view, _("Loading..."));
-	else
-		e_data_book_view_notify_status_message (book_view, _("Searching..."));
-
-	closure->view = book_view;
-	closure->bvcf = bvcf;
-
-	g_idle_add (e_book_backend_vcf_search_timeout, closure);
-}
-
 static void
 insert_contact (EBookBackendVCF *vcf, char *vcard)
 {
@@ -276,6 +223,7 @@ do_create(EBookBackendVCF  *bvcf,
 static EBookBackendSyncStatus
 e_book_backend_vcf_create_contact (EBookBackendSync *backend,
 				   EDataBook *book,
+				   guint32 opid,
 				   const char *vcard,
 				   EContact **contact)
 {
@@ -294,7 +242,8 @@ e_book_backend_vcf_create_contact (EBookBackendSync *backend,
 
 static EBookBackendSyncStatus
 e_book_backend_vcf_remove_contacts (EBookBackendSync *backend,
-				    EDataBook    *book,
+				    EDataBook *book,
+				    guint32 opid,
 				    GList *id_list,
 				    GList **ids)
 {
@@ -326,6 +275,7 @@ e_book_backend_vcf_remove_contacts (EBookBackendSync *backend,
 static EBookBackendSyncStatus
 e_book_backend_vcf_modify_contact (EBookBackendSync *backend,
 				   EDataBook *book,
+				   guint32 opid,
 				   const char *vcard,
 				   EContact **contact)
 {
@@ -361,7 +311,8 @@ e_book_backend_vcf_modify_contact (EBookBackendSync *backend,
 
 static EBookBackendSyncStatus
 e_book_backend_vcf_get_contact (EBookBackendSync *backend,
-				EDataBook    *book,
+				EDataBook *book,
+				guint32 opid,
 				const char *id,
 				char **vcard)
 {
@@ -397,7 +348,8 @@ foreach_get_contact_compare (char *id, char *vcard_string, GetContactListClosure
 
 static EBookBackendSyncStatus
 e_book_backend_vcf_get_contact_list (EBookBackendSync *backend,
-				     EDataBook    *book,
+				     EDataBook *book,
+				     guint32 opid,
 				     const char *query,
 				     GList **contacts)
 {
@@ -418,18 +370,138 @@ e_book_backend_vcf_get_contact_list (EBookBackendSync *backend,
 	return GNOME_Evolution_Addressbook_Success;
 }
 
+typedef struct {
+	EBookBackendVCF *bvcf;
+	EDataBookView *view;
+	GMutex *mutex;
+	GCond *cond;
+	GThread *thread;
+	gboolean stopped;
+} VCFBackendSearchClosure;
+
+static void
+closure_destroy (VCFBackendSearchClosure *closure)
+{
+	d(printf ("destroying search closure\n"));
+	g_mutex_free (closure->mutex);
+	g_cond_free (closure->cond);
+	g_free (closure);
+}
+
+static VCFBackendSearchClosure*
+init_closure (EDataBookView *book_view, EBookBackendVCF *bvcf)
+{
+	VCFBackendSearchClosure *closure = g_new (VCFBackendSearchClosure, 1);
+
+	closure->bvcf = bvcf;
+	closure->view = book_view;
+	closure->mutex = g_mutex_new ();
+	closure->cond = g_cond_new ();
+	closure->thread = NULL;
+	closure->stopped = FALSE;
+
+	g_object_set_data_full (G_OBJECT (book_view), "EBookBackendVCF.BookView::closure",
+				closure, (GDestroyNotify)closure_destroy);
+
+	return closure;
+}
+
+static VCFBackendSearchClosure*
+get_closure (EDataBookView *book_view)
+{
+	return g_object_get_data (G_OBJECT (book_view), "EBookBackendVCF.BookView::closure");
+}
+
+static void
+foreach_search_compare (char *id, char *vcard_string, VCFBackendSearchClosure *closure)
+{
+	EContact *contact;
+
+	/* we really need to stop the entire loop when stopped ==
+	   TRUE, but we can't loop over the hash table that way..
+	   lame.  so we just return here. */
+	if (closure->stopped)
+		return;
+
+	contact = e_contact_new_from_vcard (vcard_string);
+	e_data_book_view_notify_update (closure->view, contact);
+	g_object_unref (contact);
+}
+
+static gpointer
+book_view_thread (gpointer data)
+{
+	EDataBookView *book_view = data;
+	VCFBackendSearchClosure *closure = get_closure (book_view);
+	const char *query;
+
+	/* ref the book view because it'll be removed and unrefed
+	   when/if it's stopped */
+	bonobo_object_ref (book_view);
+
+	query = e_data_book_view_get_card_query (book_view);
+
+	if ( ! strcmp (query, "(contains \"x-evolution-any-field\" \"\")"))
+		e_data_book_view_notify_status_message (book_view, _("Loading..."));
+	else
+		e_data_book_view_notify_status_message (book_view, _("Searching..."));
+
+	d(printf ("signalling parent thread\n"));
+	g_mutex_lock (closure->mutex);
+	g_cond_signal (closure->cond);
+	g_mutex_unlock (closure->mutex);
+
+	g_hash_table_foreach (closure->bvcf->priv->contacts,
+			      (GHFunc)foreach_search_compare,
+			      closure);
+
+	if (!closure->stopped)
+		e_data_book_view_notify_complete (closure->view, GNOME_Evolution_Addressbook_Success);
+
+	/* unref the book view */
+	bonobo_object_unref (book_view);
+
+	d(printf ("finished initial population of book view\n"));
+
+	return NULL;
+}
+
+
 static void
 e_book_backend_vcf_start_book_view (EBookBackend  *backend,
-				 EDataBookView *book_view)
+				    EDataBookView *book_view)
 {
-	e_book_backend_vcf_search (E_BOOK_BACKEND_VCF (backend), book_view);
+	VCFBackendSearchClosure *closure = init_closure (book_view, E_BOOK_BACKEND_VCF (backend));
+
+	g_mutex_lock (closure->mutex);
+
+	d(printf ("starting book view thread\n"));
+	closure->thread = g_thread_create (book_view_thread, book_view, TRUE, NULL);
+
+	g_cond_wait (closure->cond, closure->mutex);
+
+	/* at this point we know the book view thread is actually running */
+	g_mutex_unlock (closure->mutex);
+	d(printf ("returning from start_book_view\n"));
+
 }
 
 static void
 e_book_backend_vcf_stop_book_view (EBookBackend  *backend,
-				EDataBookView *book_view)
+				   EDataBookView *book_view)
 {
-	/* XXX nothing here yet, and there should be. */
+	VCFBackendSearchClosure *closure = get_closure (book_view);
+	gboolean need_join = FALSE;
+
+	d(printf ("stopping query\n"));
+	g_mutex_lock (closure->mutex);
+	if (!closure->stopped)
+		need_join = TRUE;
+	closure->stopped = TRUE;
+	g_mutex_unlock (closure->mutex);
+
+	if (need_join)
+		g_thread_join (closure->thread);
 }
 
 static char *
@@ -442,7 +514,8 @@ e_book_backend_vcf_extract_path_from_uri (const char *uri)
 
 static EBookBackendSyncStatus
 e_book_backend_vcf_authenticate_user (EBookBackendSync *backend,
-				      EDataBook    *book,
+				      EDataBook *book,
+				      guint32 opid,
 				      const char *user,
 				      const char *passwd,
 				      const char *auth_method)
@@ -452,7 +525,8 @@ e_book_backend_vcf_authenticate_user (EBookBackendSync *backend,
 
 static EBookBackendSyncStatus
 e_book_backend_vcf_get_supported_fields (EBookBackendSync *backend,
-					 EDataBook    *book,
+					 EDataBook *book,
+					 guint32 opid,
 					 GList **fields_out)
 {
 	GList *fields = NULL;
@@ -467,7 +541,9 @@ e_book_backend_vcf_get_supported_fields (EBookBackendSync *backend,
 	return GNOME_Evolution_Addressbook_Success;
 }
 
-#include <libedata-book/ximian-vcard.h>
+#ifdef CREATE_DEFAULT_VCARD
+# include <libedata-book/ximian-vcard.h>
+#endif
 
 static GNOME_Evolution_Addressbook_CallStatus
 e_book_backend_vcf_load_source (EBookBackend             *backend,
@@ -495,10 +571,24 @@ e_book_backend_vcf_load_source (EBookBackend             *backend,
 	} else {
 		fd = open (bvcf->priv->filename, O_RDONLY);
 
-		if (fd == -1) {
+		if (fd == -1 && !only_if_exists) {
+			int rv;
+
+			/* the database didn't exist, so we create the
+			   directory then the .vcf file */
+			rv = e_util_mkdir_hier (dirname, 0777);
+			if (rv == -1 && errno != EEXIST) {
+				g_warning ("failed to make directory %s: %s", dirname, strerror (errno));
+				if (errno == EACCES || errno == EPERM)
+					return GNOME_Evolution_Addressbook_PermissionDenied;
+				else
+					return GNOME_Evolution_Addressbook_OtherError;
+			}
+
 			fd = open (bvcf->priv->filename, O_CREAT, 0666);
 
-			if (fd != -1 && !only_if_exists) {
+			if (fd != -1) {
+#ifdef CREATE_DEFAULT_VCARD
 				EContact *contact;
 
 				contact = do_create(bvcf, XIMIAN_VCARD, FALSE);
@@ -506,6 +596,7 @@ e_book_backend_vcf_load_source (EBookBackend             *backend,
 
 				/* XXX check errors here */
 				g_object_unref (contact);
+#endif
 
 				writable = TRUE;
 			}
