@@ -16,19 +16,24 @@
 #include "e-book-backend-sexp.h"
 #include "e-data-book-view.h"
 
+#define d(x)
+
 static BonoboObjectClass *e_data_book_view_parent_class;
 
 struct _EDataBookViewPrivate {
 	GNOME_Evolution_Addressbook_BookViewListener  listener;
 
-#define INITIAL_THRESHOLD 20
-#define THRESHOLD_MAX 3000
+#define DEFAULT_INITIAL_THRESHOLD 20
+#define DEFAULT_THRESHOLD_MAX 3000
+
+	GMutex *mutex;
 
 	GMutex *pending_mutex;
 
 	CORBA_sequence_GNOME_Evolution_Addressbook_VCard adds;
 	int next_threshold;
 	int threshold_max;
+	int threshold_min;
 
 	CORBA_sequence_GNOME_Evolution_Addressbook_VCard changes;
 	CORBA_sequence_GNOME_Evolution_Addressbook_ContactId removes;
@@ -37,8 +42,21 @@ struct _EDataBookViewPrivate {
 	char *card_query;
 	EBookBackendSExp *card_sexp;
 	GHashTable *ids;
+
+	int max_results;
 };
 
+static void
+view_listener_died_cb (gpointer cnx, gpointer user_data)
+{
+	EDataBookView *book_view = E_DATA_BOOK_VIEW (user_data);
+
+	if (book_view) {
+		e_book_backend_stop_book_view (e_data_book_view_get_backend (book_view), book_view);
+		bonobo_object_unref (book_view);
+	}
+}
+
 static void
 send_pending_adds (EDataBookView *book_view, gboolean reset)
 {
@@ -66,7 +84,7 @@ send_pending_adds (EDataBookView *book_view, gboolean reset)
 	adds->_length = 0;
 
 	if (reset)
-		book_view->priv->next_threshold = INITIAL_THRESHOLD;
+		book_view->priv->next_threshold = book_view->priv->threshold_min;
 }
 
 static void
@@ -304,10 +322,11 @@ e_data_book_view_notify_status_message (EDataBookView *book_view,
 
 static void
 e_data_book_view_construct (EDataBookView                *book_view,
-			 EBookBackend                 *backend,
-			 GNOME_Evolution_Addressbook_BookViewListener  listener,
-			 const char                 *card_query,
-			 EBookBackendSExp         *card_sexp)
+			    EBookBackend                 *backend,
+			    GNOME_Evolution_Addressbook_BookViewListener  listener,
+			    const char                   *card_query,
+			    EBookBackendSExp             *card_sexp,
+			    int                           max_results)
 {
 	EDataBookViewPrivate *priv;
 	CORBA_Environment ev;
@@ -319,7 +338,7 @@ e_data_book_view_construct (EDataBookView                *book_view,
 
 	CORBA_exception_init (&ev);
 
-	priv->listener = CORBA_Object_duplicate (listener, &ev);
+	priv->listener = bonobo_object_dup_ref (listener, &ev);
 	if (ev._major != CORBA_NO_EXCEPTION) {
 		g_warning("Unable to duplicate listener object in pas-book-view.c\n");
 		CORBA_exception_free (&ev);
@@ -331,6 +350,9 @@ e_data_book_view_construct (EDataBookView                *book_view,
 	priv->backend = backend;
 	priv->card_query = g_strdup (card_query);
 	priv->card_sexp = card_sexp;
+	priv->max_results = max_results;
+
+	ORBit_small_listen_for_broken (e_data_book_view_get_listener (book_view), G_CALLBACK (view_listener_died_cb), book_view);
 }
 
 /**
@@ -354,12 +376,26 @@ impl_GNOME_Evolution_Addressbook_BookView_stop (PortableServer_Servant servant,
 	e_book_backend_stop_book_view (e_data_book_view_get_backend (view), view);
 }
 
+static void
+impl_GNOME_Evolution_Addressbook_BookView_dispose (PortableServer_Servant servant,
+						   CORBA_Environment *ev)
+{
+	EDataBookView *view = E_DATA_BOOK_VIEW (bonobo_object (servant));
+
+	d(printf("in impl_GNOME_Evolution_Addressbook_BookView_dispose\n"));
+	ORBit_small_unlisten_for_broken (e_data_book_view_get_listener (view), G_CALLBACK (view_listener_died_cb));
+
+	bonobo_object_unref (view);
+}
+
 /**
  * e_data_book_view_get_card_query
  */
 const char*
 e_data_book_view_get_card_query (EDataBookView *book_view)
 {
+	g_return_val_if_fail (book_view, NULL);
+
 	return book_view->priv->card_query;
 }
 
@@ -369,37 +405,69 @@ e_data_book_view_get_card_query (EDataBookView *book_view)
 EBookBackendSExp*
 e_data_book_view_get_card_sexp (EDataBookView *book_view)
 {
+	g_return_val_if_fail (E_IS_DATA_BOOK_VIEW (book_view), NULL);
+
 	return book_view->priv->card_sexp;
 }
 
+int
+e_data_book_view_get_max_results (EDataBookView *book_view)
+{
+	g_return_val_if_fail (E_IS_DATA_BOOK_VIEW (book_view), 0);
+
+	return book_view->priv->max_results;
+}
+
 EBookBackend*
 e_data_book_view_get_backend (EDataBookView *book_view)
 {
+	g_return_val_if_fail (E_IS_DATA_BOOK_VIEW (book_view), NULL);
+
 	return book_view->priv->backend;
 }
 
+GMutex*
+e_data_book_view_get_mutex (EDataBookView *book_view)
+{
+	g_return_val_if_fail (E_IS_DATA_BOOK_VIEW (book_view), NULL);
+
+	return book_view->priv->mutex;
+}
+
 GNOME_Evolution_Addressbook_BookViewListener
 e_data_book_view_get_listener (EDataBookView  *book_view)
 {
+	g_return_val_if_fail (E_IS_DATA_BOOK_VIEW (book_view), CORBA_OBJECT_NIL);
+
 	return book_view->priv->listener;
 }
 
+void
+e_data_book_view_set_thresholds (EDataBookView *book_view,
+				 int minimum_grouping_threshold,
+				 int maximum_grouping_threshold)
+{
+	book_view->priv->threshold_min = minimum_grouping_threshold;
+	book_view->priv->threshold_max = maximum_grouping_threshold;
+}
+
 /**
  * e_data_book_view_new:
  */
 EDataBookView *
 e_data_book_view_new (EBookBackend *backend,
-		   GNOME_Evolution_Addressbook_BookViewListener  listener,
-		   const char *card_query,
-		   EBookBackendSExp *card_sexp)
+		      GNOME_Evolution_Addressbook_BookViewListener  listener,
+		      const char *card_query,
+		      EBookBackendSExp *card_sexp,
+		      int max_results)
 {
 	EDataBookView *book_view;
 
 	book_view = g_object_new (E_TYPE_DATA_BOOK_VIEW,
-				  "poa", bonobo_poa_get_threaded (ORBIT_THREAD_HINT_PER_REQUEST, NULL),
+				  "poa", bonobo_poa_get_threaded (ORBIT_THREAD_HINT_PER_OBJECT, NULL),
 				  NULL);
 	
-	e_data_book_view_construct (book_view, backend, listener, card_query, card_sexp);
+	e_data_book_view_construct (book_view, backend, listener, card_query, card_sexp, max_results);
 
 	return book_view;
 }
@@ -409,6 +477,7 @@ e_data_book_view_dispose (GObject *object)
 {
 	EDataBookView *book_view = E_DATA_BOOK_VIEW (object);
 
+	d(printf ("disposing of EDataBookView\n"));
 	if (book_view->priv) {
 		bonobo_object_release_unref (book_view->priv->listener, NULL);
 
@@ -423,7 +492,10 @@ e_data_book_view_dispose (GObject *object)
 		g_object_unref (book_view->priv->card_sexp);
 
 		g_mutex_free (book_view->priv->pending_mutex);
-		book_view->priv->pending_mutex = NULL;
+
+		g_mutex_free (book_view->priv->mutex);
+
+		g_hash_table_destroy (book_view->priv->ids);
 
 		g_free (book_view->priv);
 		book_view->priv = NULL;
@@ -447,6 +519,7 @@ e_data_book_view_class_init (EDataBookViewClass *klass)
 
 	epv->start                = impl_GNOME_Evolution_Addressbook_BookView_start;
 	epv->stop                 = impl_GNOME_Evolution_Addressbook_BookView_stop;
+	epv->dispose              = impl_GNOME_Evolution_Addressbook_BookView_dispose;
 
 }
 
@@ -456,9 +529,11 @@ e_data_book_view_init (EDataBookView *book_view)
 	book_view->priv = g_new0 (EDataBookViewPrivate, 1);
 
 	book_view->priv->pending_mutex = g_mutex_new();
+	book_view->priv->mutex = g_mutex_new();
 
-	book_view->priv->next_threshold = INITIAL_THRESHOLD;
-	book_view->priv->threshold_max = THRESHOLD_MAX;
+	book_view->priv->threshold_min = DEFAULT_INITIAL_THRESHOLD;
+	book_view->priv->threshold_max = DEFAULT_THRESHOLD_MAX;
+	book_view->priv->next_threshold = book_view->priv->threshold_min;
 
 	book_view->priv->ids = g_hash_table_new_full (g_str_hash, g_str_equal,
 						      g_free, NULL);
