@@ -8,10 +8,14 @@ int main (int argc, char **argv)
 
 	gnome_program_init("test-nonexistent-id", "0.0", LIBGNOME_MODULE, argc, argv, NULL);
 
-	book = e_book_new ();
-
 	printf ("loading addressbook\n");
-	if (!e_book_load_local_addressbook (book, NULL)) {
+	book = e_book_new_system_addressbook (NULL);
+	if (!book) {
+		printf ("failed to create local addressbook\n");
+		exit(0);
+	}
+
+	if (!e_book_open (book, FALSE, NULL)) {
 		printf ("failed to open local addressbook\n");
 		exit(0);
 	}
