@@ -63,6 +63,9 @@ struct _ECalBackendHttpPrivate {
 	/* Reload */
 	guint reload_timeout_id;
 	guint is_loading : 1;
+
+	/* Flags */
+	gboolean opened;
 };
 
 
@@ -217,8 +220,10 @@ retrieval_done (SoupMessage *msg, ECalBackendHttp *cbhttp)
 			priv->uri = webcal_to_http_method (newuri);
 			begin_retrieval_cb (cbhttp);
 		} else {
-			e_cal_backend_notify_error (E_CAL_BACKEND (cbhttp),
-						    _("Redirected to Invalid URI"));
+			if (!priv->opened) {
+				e_cal_backend_notify_error (E_CAL_BACKEND (cbhttp),
+							    _("Redirected to Invalid URI"));
+			}
 		}
 
 		return;
@@ -226,8 +231,10 @@ retrieval_done (SoupMessage *msg, ECalBackendHttp *cbhttp)
 
 	/* check status code */
 	if (!SOUP_STATUS_IS_SUCCESSFUL (msg->status_code)) {
-		e_cal_backend_notify_error (E_CAL_BACKEND (cbhttp),
-					    soup_status_get_phrase (msg->status_code));
+		if (!priv->opened) {
+			e_cal_backend_notify_error (E_CAL_BACKEND (cbhttp),
+						    soup_status_get_phrase (msg->status_code));
+		}
 		return;
 	}
 
@@ -238,31 +245,47 @@ retrieval_done (SoupMessage *msg, ECalBackendHttp *cbhttp)
 	g_free (str);
 
 	if (!icalcomp) {
-		e_cal_backend_notify_error (E_CAL_BACKEND (cbhttp), _("Bad file format."));
+		if (!priv->opened)
+			e_cal_backend_notify_error (E_CAL_BACKEND (cbhttp), _("Bad file format."));
 		return;
 	}
 
 	if (icalcomponent_isa (icalcomp) != ICAL_VCALENDAR_COMPONENT) {
-		e_cal_backend_notify_error (E_CAL_BACKEND (cbhttp), _("Not a calendar."));
+		if (!priv->opened)
+			e_cal_backend_notify_error (E_CAL_BACKEND (cbhttp), _("Not a calendar."));
 		icalcomponent_free (icalcomp);
 		return;
 	}
 
 	/* Update cache */
+	e_file_cache_clean (E_FILE_CACHE (priv->cache));
+
 	kind = e_cal_backend_get_kind (E_CAL_BACKEND (cbhttp));
-	subcomp = icalcomponent_get_first_component (icalcomp, kind);
+	subcomp = icalcomponent_get_first_component (icalcomp, ICAL_ANY_COMPONENT);
 	while (subcomp) {
 		ECalComponent *comp;
+		icalcomponent_kind subcomp_kind;
 
-		comp = e_cal_component_new ();
-		if (e_cal_component_set_icalcomponent (comp, subcomp)) {
-			e_cal_backend_cache_put_component (priv->cache, comp);
-			e_cal_backend_notify_object_created (E_CAL_BACKEND (cbhttp),
-							     icalcomponent_as_ical_string (subcomp));
+		subcomp_kind = icalcomponent_isa (subcomp);
+		if (subcomp_kind == kind) {
+			comp = e_cal_component_new ();
+			if (e_cal_component_set_icalcomponent (comp, subcomp)) {
+				e_cal_backend_cache_put_component (priv->cache, comp);
+				e_cal_backend_notify_object_created (E_CAL_BACKEND (cbhttp),
+								     icalcomponent_as_ical_string (subcomp));
+			}
+
+			g_object_unref (comp);
+		} else if (subcomp_kind == ICAL_VTIMEZONE_COMPONENT) {
+			icaltimezone *zone;
+
+			zone = icaltimezone_new ();
+			icaltimezone_set_component (zone, subcomp);
+			e_cal_backend_cache_put_timezone (priv->cache, (const icaltimezone *) zone);
+
+			icaltimezone_free (zone, 1);
 		}
 
-		g_object_unref (comp);
-
 		subcomp = icalcomponent_get_next_component (icalcomp, kind);
 	}
 
@@ -327,6 +350,7 @@ reload_cb (ECalBackendHttp *cbhttp)
 	d(g_message ("Reload!\n"));
 
 	priv->reload_timeout_id = 0;
+	priv->opened = TRUE;
 	begin_retrieval_cb (cbhttp);
 	return FALSE;
 }
@@ -809,6 +833,7 @@ e_cal_backend_http_init (ECalBackendHttp *cbhttp, ECalBackendHttpClass *class)
 
 	priv->uri = NULL;
 	priv->reload_timeout_id = 0;
+	priv->opened = FALSE;
 }
 
 /* Class initialization function for the file backend */
